const https = require('https');
const lighthouse = require('lighthouse');
const chromeLauncher = require('chrome-launcher');
const pkg = require(process.cwd() + '/package.json');
const defaultConfig = {
  "minimum-scores": {
    "accessibility": .75,
    "best-practices": .75,
    "performance": .75,
    "pwa": .75,
    "seo": .75
  },
  "urls": []
};
const config = Object.assign(defaultConfig, pkg['lighthouse-tests']);

function launchChromeAndRunLighthouse(url, opts, config = null) {
  return chromeLauncher.launch({chromeFlags: opts.chromeFlags}).then(chrome => {
    opts.port = chrome.port;
    return lighthouse(url, opts, config).then(results => {
      return chrome.kill().then(() => results.lhr)
    });
  });
}

const opts = {};
const categories = Object.keys(config['minimum-scores']);


config.urls.forEach(url => {
  const request = https.request(url, {}, res => {
    launchChromeAndRunLighthouse(url, opts).then(results => {
      const errors = [];

      categories.forEach(category => {
        if (results.categories[category].score < config['minimum-scores'][category]) {
          errors.push(`${category} score not met: ${results.categories[category].score} (should be at least ${config['minimum-scores'][category]})`);
        }
      });

      if (errors.length !== 0) {
        console.error(`Errors found for ${results.requestedUrl}:`);
        errors.forEach(err => {
          console.error(err);
        })
      }
    });
  });

  request.on('error', (e) => {
    console.error(`problem with request: ${e.message}`);
  });

  request.end();
})
